@@ -21,10 +21,10 @@ struct lsCodeLens {
         // The range in which this code lens is valid. Should only span a single line.
         lsRange range;
         // The command this code lens represents.
-        boost::optional<lsCommandWithAny> command;
+        optional<lsCommandWithAny> command;
         // A data entry field that is preserved on a code lens item between
         // a code lens and a code lens resolve request.
-        boost::optional< lsp::Any> data;
+        optional< lsp::Any> data;
 
         MAKE_SWAP_METHOD(lsCodeLens, range, command, data)
 };
