@@ -3,7 +3,7 @@
 
 GCThreadContext::GCThreadContext()
 {
-#ifdef USEGC
+#ifdef LSPCPP_USEGC
     GC_get_stack_base(&gsb);
     GC_register_my_thread(&gsb);
 #endif
@@ -11,7 +11,7 @@ GCThreadContext::GCThreadContext()
 
 GCThreadContext::~GCThreadContext()
 {
-#ifdef USEGC
+#ifdef LSPCPP_USEGC
     GC_unregister_my_thread();
 #endif
 }
\ No newline at end of file
