@@ -239,7 +239,7 @@ namespace lsp {
         WebSocketServer::WebSocketServer(const std::string& user_agent, const std::string& address, const std::string& port,
             std::shared_ptr < MessageJsonHandler> json_handler,
             std::shared_ptr < Endpoint> localEndPoint, lsp::Log& log, uint32_t _max_workers)
-            : point(json_handler,localEndPoint,log, _max_workers),d_ptr(new Data(user_agent,log))
+            : point(json_handler,localEndPoint,log,lsp::JSONStreamStyle::Standard, _max_workers),d_ptr(new Data(user_agent,log))
 
         {
 
