@@ -231,7 +231,7 @@ void Reflect(Reader& visitor, TextDocumentDocumentSymbol::Either& value)
         }
 }
 
-void Reflect(Reader& visitor, std::pair<boost::optional<std::string>, boost::optional<lsMarkedString>>& value)
+void Reflect(Reader& visitor, std::pair<optional<std::string>, optional<lsMarkedString>>& value)
 {
 
         if (!visitor.IsString())
@@ -244,7 +244,7 @@ void Reflect(Reader& visitor, std::pair<boost::optional<std::string>, boost::opt
         }
 }
 
-void Reflect(Reader& visitor, std::pair<boost::optional<std::string>, boost::optional<MarkupContent>>& value)
+void Reflect(Reader& visitor, std::pair<optional<std::string>, optional<MarkupContent>>& value)
 {
         if (!visitor.IsString())
         {
