@@ -9,36 +9,33 @@
 
 #undef NDEBUG
 
+#if defined(_WIN32)
+#include <Winsock2.h>
+#endif
+
 #include <iostream>
 #include <memory>
 #include <climits>
+#include <cstdint>
+
+#if defined(_MSC_VER)
+// for and/or/not operators. not supported natively on MSVC
+#include <BaseTsd.h>
+#include <ciso646>
+typedef SSIZE_T ssize_t;
+#define STDOUT_FILENO 1
+#define STDIN_FILENO 0
+#define STDERR_FILENO 2
+#endif
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#if __cplusplus < 201703L
-
-#ifdef HAVE_LSP
-#include <boost/optional.hpp>
-#include <boost/none.hpp>
-using boost::optional;
-#define nullopt boost::none
-using boost::make_optional;
-#else
-#include "optional.hpp"
-#define boost nonstd
+#include <optional.hpp>
 using nonstd::optional;
 using nonstd::nullopt;
 using nonstd::make_optional;
-#endif
-
-#else
-#include <optional>
-using std::optional;
-using std::nullopt;
-using std::make_optional;
-#endif
 
 using std::make_pair;
 
@@ -58,26 +55,10 @@ using std::make_pair;
 
 #include "memory.h"
 
-#if defined(HAVE_LONG_LONG) && defined(LLONG_MAX) && defined(LLONG_MIN)
-#define Int_MAX2 LLONG_MAX
-#define Int_MIN LLONG_MIN
-typedef long long Int;
-typedef unsigned long long unsignedInt;
-#else
-#undef HAVE_LONG_LONG
-#ifdef HAVE_LONG
-#define Int_MAX2 LONG_MAX
-#define Int_MIN LONG_MIN
-typedef long Int;
-typedef unsigned long unsignedInt;
-#else
-#define Int_MAX2 INT_MAX
-#define Int_MIN INT_MIN
-#define intIsInt
-typedef int Int;
-typedef unsigned int unsignedInt;
-#endif
-#endif
+#define Int_MAX2 INT64_MAX
+#define Int_MIN INT64_MIN
+typedef int64_t Int;
+typedef uint64_t unsignedInt;
 
 #ifndef COMPACT
 #if Int_MAX2 >= 0x7fffffffffffffffLL
