@@ -45,9 +45,9 @@ using sym::symbol;
 
 class vardec;
 
-class ty : public absyn {
+class astType : public absyn {
 public:
-  ty(position pos)
+  astType(position pos)
     : absyn(pos) {}
 
   virtual void prettyprint(ostream &out, Int indent) = 0;
@@ -65,19 +65,21 @@ public:
 
   virtual operator string() const = 0;
 #ifdef USEGC
-  operator std::string() const { return mem::stdString(this->operator string()); }
+  operator std::string() const {
+    return mem::stdString(this->operator string());
+  }
 #endif
 };
 
-class nameTy : public ty {
+class nameTy : public astType {
   name *id;
 
 public:
   nameTy(position pos, name *id)
-    : ty(pos), id(id) {}
+    : astType(pos), id(id) {}
 
   nameTy(name *id)
-    : ty(id->getPos()), id(id) {}
+    : astType(id->getPos()), id(id) {}
 
   void prettyprint(ostream &out, Int indent) override;
 
@@ -102,19 +104,19 @@ public:
     return depth;
   }
 
-  types::array *truetype(types::ty *base);
+  types::array *truetype(types::ty *base, bool tacit=false);
 };
 
-class arrayTy : public ty {
-  ty *cell;
+class arrayTy : public astType {
+  astType *cell;
   dimensions *dims;
 
 public:
-  arrayTy(position pos, ty *cell, dimensions *dims)
-    : ty(pos), cell(cell), dims(dims) {}
+  arrayTy(position pos, astType *cell, dimensions *dims)
+    : astType(pos), cell(cell), dims(dims) {}
 
   arrayTy(name *id, dimensions *dims)
-    : ty(dims->getPos()), cell(new nameTy(id)), dims(dims) {}
+    : astType(dims->getPos()), cell(new nameTy(id)), dims(dims) {}
 
   void prettyprint(ostream &out, Int indent) override;
 
@@ -125,13 +127,13 @@ public:
   operator string() const override;
 };
 
-// Similar to varEntryExp, this helper class always translates to the same fixed
-// type.
-class tyEntryTy : public ty {
+// Similar to varEntryExp, this helper class always translates to the same
+// fixed type.
+class tyEntryTy : public astType {
   trans::tyEntry *ent;
 public:
   tyEntryTy(position pos, trans::tyEntry *ent)
-    : ty(pos), ent(ent) {}
+    : astType(pos), ent(ent) {}
 
   tyEntryTy(position pos, types::ty *t);
 
@@ -197,6 +199,19 @@ public:
   { return false; }
 };
 
+// Forward declaration.
+class formals;
+
+class namedTyEntry : public gc {
+public:
+  symbol dest;
+  trans::tyEntry *ent;
+  position pos;
+  namedTyEntry(position pos, symbol dest, trans::tyEntry *ent)
+    : dest(dest), ent(ent), pos(pos) {}
+};
+
+
 class block : public runnable {
 public:
   mem::list<runnable *> stms;
@@ -224,10 +239,26 @@ public:
 
   void transAsField(coenv &e, record *r) override;
 
+  bool transAsTemplatedField(
+    coenv &e, record *r, mem::vector<absyntax::namedTyEntry*>* args,
+    trans::frame *caller
+  );
+
   void transAsRecordBody(coenv &e, record *r);
+  bool transAsTemplatedRecordBody(
+    coenv &e, record *r, mem::vector<absyntax::namedTyEntry*> *args,
+    trans::frame *caller
+  );
 
   types::record *transAsFile(genv& ge, symbol id);
 
+  types::record *transAsTemplatedFile(
+      genv& ge,
+      symbol id,
+      mem::vector<absyntax::namedTyEntry*> *args,
+      coenv &e
+  );
+
   // If the block can be interpreted as a single vardec, return that vardec
   // (otherwise 0).
   vardec *asVardec();
@@ -333,12 +364,11 @@ public:
   { return id; }
 
   void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
-  void createSymMapWType(AsymptoteLsp::SymbolContext* symContext, absyntax::ty* base);
+  void createSymMapWType(
+      AsymptoteLsp::SymbolContext* symContext, absyntax::astType* base
+  );
 };
 
-// Forward declaration.
-class formals;
-
 class fundecidstart : public decidstart {
   formals *params;
 
@@ -377,7 +407,9 @@ public:
   decidstart *getStart() { return start; }
 
   void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
-  void createSymMapWType(AsymptoteLsp::SymbolContext* symContext, absyntax::ty* base);
+  void createSymMapWType(
+      AsymptoteLsp::SymbolContext* symContext, absyntax::astType* base
+  );
 };
 
 class decidlist : public absyn {
@@ -410,7 +442,9 @@ public:
   }
 
   void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
-  void createSymMapWType(AsymptoteLsp::SymbolContext* symContext, absyntax::ty* base);
+  void createSymMapWType(
+      AsymptoteLsp::SymbolContext* symContext, absyntax::astType* base
+  );
 };
 
 class dec : public runnable {
@@ -429,14 +463,14 @@ void createVar(position pos, coenv &e, record *r,
                symbol id, types::ty *t, varinit *init);
 
 class vardec : public dec {
-  ty *base;
+  astType *base;
   decidlist *decs;
 
 public:
-  vardec(position pos, ty *base, decidlist *decs)
+  vardec(position pos, astType *base, decidlist *decs)
     : dec(pos), base(base), decs(decs) {}
 
-  vardec(position pos, ty *base, decid *di)
+  vardec(position pos, astType *base, decid *di)
     : dec(pos), base(base), decs(new decidlist(pos))
   {
     decs->add(di);
@@ -543,6 +577,86 @@ public:
   void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
 };
 
+class badDec : public dec {
+  position errorPos;
+  string errorMessage;
+
+public:
+  badDec(position pos, position errorPos, string errorMessage)
+      : dec(pos), errorPos(errorPos), errorMessage(errorMessage) {}
+
+  void transAsField(coenv&, record*) override {
+    em.error(errorPos);
+    em << errorMessage;
+  }
+};
+
+// Accesses the file with specified types added to the type environment.
+class templateAccessDec : public dec {
+  symbol src; // The name of the module to access.
+  formals *args;
+  symbol dest;  // What to call it in the local environment.
+  bool valid;
+  position expectedAsPos;
+
+public:
+  templateAccessDec(position pos, symbol src, formals* args, symbol as,
+                    symbol dest, position asPos)
+      : dec(pos), src(src), args(args), dest(dest),
+        valid(as == symbol::trans("as")), expectedAsPos(asPos) {}
+
+  bool checkValidity() {
+    if (!valid) {
+      em.error(expectedAsPos);
+      em << "expected 'as'";
+      return false;
+    }
+    return true;
+  }
+
+  void transAsField(coenv& e, record* r) override;
+};
+
+class typeParam : public absyn {
+  const symbol paramSym;
+public:
+  typeParam(position pos, symbol paramSym)
+    : absyn(pos), paramSym(paramSym) {}
+
+  bool transAsParamMatcher(coenv &e, record *r, namedTyEntry *arg);
+
+  void prettyprint(ostream &out, Int indent);
+};
+
+class typeParamList : public absyn {
+  mem::vector<typeParam*> params;
+
+public:
+  typeParamList(position pos) : absyn(pos) {}
+
+  void add(typeParam *tp);
+
+  bool transAsParamMatcher(coenv &e, record *r,
+                           mem::vector<namedTyEntry*> *args, trans::frame *caller);
+
+  void prettyprint(ostream &out, Int indent);
+};
+
+
+class receiveTypedefDec : public dec {
+  typeParamList* params;
+
+public:
+  receiveTypedefDec(position pos, typeParamList* params)
+    : dec(pos), params(params) {}
+
+  void transAsField(coenv& e, record *r) override;
+  bool transAsParamMatcher(
+    coenv& e, record *r, mem::vector<namedTyEntry*> *args, trans::frame *caller
+  );
+};
+
+
 // Abstract base class for
 //   from _ access _;  (fromaccessdec)
 // and
@@ -551,8 +665,8 @@ class fromdec : public dec {
 protected:
   struct qualifier {
     // The varEntry holds the location and the type of the highest framed
-    // structure that can be put on the stack.  The record holds the actual type
-    // of the qualifier.
+    // structure that can be put on the stack.  The record holds the actual
+    // type of the qualifier.
     // For example:
     //   struct A {
     //     struct B {
@@ -563,7 +677,7 @@ protected:
     //   from a.B unravel x;
     //
     // Here, v->getType() will yield A and v->getLocation() will yield the
-    // location of the the variable a, but the record type t will be B.
+    // location of the variable a, but the record type t will be B.
     record *t;
     varEntry *v;
 
@@ -604,11 +718,13 @@ public:
 // scope.  It does not add the module as a variable in the local scope.
 class fromaccessdec : public fromdec {
   symbol id;
+  formals *templateArgs;
 
   qualifier getQualifier(coenv &e, record *r) override;
 public:
-  fromaccessdec(position pos, symbol id, idpairlist *fields)
-    : fromdec(pos, fields), id(id) {}
+  fromaccessdec(
+      position pos, symbol id, idpairlist *fields, formals *templateArgs = 0
+  ) : fromdec(pos, fields), id(id), templateArgs(templateArgs) {}
 
   void prettyprint(ostream &out, Int indent) override;
 
