@@ -30,6 +30,7 @@ using types::function;
 using types::record;
 
 class genv;
+class coenv;
 
 // Keeps track of the name bindings of variables and types.  This is used for
 // the fields of a record, whereas the derived class env is used for
@@ -185,12 +186,11 @@ public:
   ~env();
 
   record *getModule(symbol id, string filename);
-  record *getTemplatedModule(symbol id,
+  record *getTemplatedModule(symbol index,
                              string filename,
-                             string index,
                              mem::vector<absyntax::namedTyEntry*> *args,
-                             trans::frame *parent
-                            );
+                             coenv& e);
+  record *getLoadedModule(symbol id);
 };
 
 } // namespace trans
