@@ -22,7 +22,7 @@
 #include "stm.h"
 #include "inst.h"
 #include "opsymbols.h"
-#include "process.h"
+#include "asyprocess.h"
 
 //void runCode(absyntax::block *code);
 
@@ -103,7 +103,7 @@ void exp::testCachedType(coenv &e) {
       em.compiler(getPos());
       em << "cached type '" << *ct
          << "' doesn't match actual type '" << *t << "'";
-      em.sync();
+      em.sync(true);
     }
   }
 }
