@@ -11,12 +11,12 @@
 # Written by Supakorn "Jamie" Rassameemasmuang <jamievlin@outlook.com>
 
 from typing import List, Tuple, Any, Union
-from datetime import datetime
+from datetime import datetime, timezone
+import os
+import time
 import io
 import argparse
-import os
 import sys
-import time
 import re
 
 def cleanComment(s):
@@ -50,17 +50,14 @@ def create_enums(filename: str) -> List[Union[Tuple[str, int, str], Tuple[str, i
 
 
 def datetime_now():
-    return datetime.utcfromtimestamp(
-        int(os.environ.get('SOURCE_DATE_EPOCH', time.time()))
-    )
-
+    return datetime.fromtimestamp(
+        int(os.environ.get('SOURCE_DATE_EPOCH', time.time())),tz=timezone.utc)
 
 def generate_enum_cpp(outname, enums, name, comment=None, *args, **kwargs):
     with io.open(outname, 'w') as fil:
         fil.write('// Enum class for {0}\n'.format(name))
         if comment is not None:
             fil.write('// {0}\n'.format(comment))
-        fil.write('// Generated at {0}\n\n'.format(datetime_now()))
         if 'namespace' in kwargs:
             fil.write('namespace {0}\n'.format(kwargs['namespace']))
             fil.write('{\n')
@@ -69,11 +66,11 @@ def generate_enum_cpp(outname, enums, name, comment=None, *args, **kwargs):
         fil.write('{\n')
 
         for enumTxt, enumNum, *ar in enums:
-            fil.write('{0}={1},\n'.format(enumTxt, enumNum))
             if len(ar) > 0:
                 comment=cleanComment(ar[-1])
                 if comment is not None:
-                    fil.write('// {0}\n\n'.format(comment.strip()))
+                    fil.write('// {0}\n'.format(comment.strip()))
+            fil.write('{0}={1},\n\n'.format(enumTxt, enumNum))
 
         fil.write('};\n\n')
 
@@ -87,8 +84,6 @@ def generate_enum_java(outname, enums, name, comment=None, *args, **kwargs):
         fil.write('// Enum class for {0}\n'.format(name))
         if comment is not None:
             fil.write('// {0}\n'.format(comment))
-        fil.write('// Generated at {0}\n\n'.format(datetime_now()))
-
         if 'package' in kwargs:
             fil.write('package {0};\n'.format(kwargs['package']))
         fil.write('\n')
@@ -130,8 +125,6 @@ def generate_enum_asy(outname, enums, name, comment=None, *args, **kwargs):
         fil.write('// Enum class for {0}\n'.format(name))
         if comment is not None:
             fil.write('// {0}\n'.format(comment))
-        fil.write('// Generated at {0}\n\n'.format(datetime_now()))
-
         fil.write('struct {0}\n'.format(name))
         fil.write('{\n')
 
@@ -153,7 +146,6 @@ def generate_enum_py(outname, enums, name, comment=None, *args, **kwargs):
         fil.write('# Enum class for {0}\n'.format(name))
         if comment is not None:
             fil.write('""" {0} """\n'.format(comment))
-        fil.write('# Generated at {0}\n\n'.format(datetime_now()))
         fil.write('class {0}:\n'.format(name))
         for enumTxt, enumNum, *ar in enums:
             fil.write('    {0}_{2}={1}\n'.format(name, enumNum, enumTxt))
