@@ -7,13 +7,31 @@
 #
 #####
 
-$prefix = shift(@ARGV);
-if (not $prefix) {
-    print STDERR "usage: ./runtime.pl module_name\n";
-    exit(1);
-}
+use strict;
+use warnings;
 
-$stack = "Stack";
+use Getopt::Long;
+
+my $opsymbolsFile;
+my $runtimeBaseFile;
+my $prefix;
+my $srcTemplateDir;
+my $headerOutDir;
+my $srcOutDir;
+
+GetOptions(
+    "opsym-file=s"        => \$opsymbolsFile,
+    "runtime-base-file=s" => \$runtimeBaseFile,
+    "src-template-dir=s"  => \$srcTemplateDir,
+    "prefix=s"            => \$prefix,
+    "header-out-dir=s"    => \$headerOutDir,
+    "src-out-dir=s"       => \$srcOutDir
+) || die("Argument error");
+
+my $outHeaderFile = "$headerOutDir/$prefix.h";
+my $outSrcFile = "$srcOutDir/$prefix.cc";
+
+my $stack = "Stack";
 
 my $errors = 0;
 
@@ -72,14 +90,16 @@ sub read_types {
 
 # Scrape the symbol names of the operators from opsymbols.h.
 my %opsymbols = ();
-open(opsyms, "opsymbols.h") ||
-        die("Couldn't open opsymbols.h");
-while (<opsyms>) {
+open(my $opsyms, $opsymbolsFile) ||
+        die("Couldn't open $opsymbolsFile");
+while (<$opsyms>) {
     if (m/^OPSYMBOL\(\"(.*)\", ([A-Za-z_]+)\);/) {
         $opsymbols{ $1 } = $2;
     }
 }
 
+close($opsyms);
+
 # Turn a name into a symbol.
 sub symbolize {
     my $name = shift;
@@ -134,9 +154,9 @@ sub c_params {
 
 $/ = "\f\n";
 
-open STDIN, "<$prefix.in" or die "can't open input file $prefix.in";
-open BASE, "<runtimebase.in" or die "can't open runtimebase.in";
-open STDOUT, ">$prefix.cc" or die "can't open output file $prefix.cc";
+open STDIN, "<$srcTemplateDir/$prefix.in" or die "can't open input file $srcTemplateDir/$prefix.in";
+open BASE, "<$runtimeBaseFile" or die "can't open $runtimeBaseFile";
+open STDOUT, ">$outSrcFile" or die "can't open output file $outSrcFile";
 
 binmode STDIN, ":unix:crlf";
 binmode BASE, ":unix:crlf";
@@ -149,25 +169,25 @@ my $source_line = 1;
 
 print $autogenerated;
 
-print "#line $base_source_line \"runtimebase.in\"\n";
-$baseheader = <BASE>;
+print "#line $base_source_line \"$srcTemplateDir/runtimebase.in\"\n";
+my $baseheader = <BASE>;
 print $baseheader;
-$basesource_line += ($baseheader =~ tr/\n//);;
+my $basesource_line += ($baseheader =~ tr/\n//);;
 my $basesource_type_line = $basesource_line;
 
-print "#line $source_line \"$prefix.in\"\n";
-$header = <>;
+print "#line $source_line \"$srcTemplateDir/$prefix.in\"\n";
+my $header = <>;
 print $header;
 $source_line += ($header =~ tr/\n//);;
 my $source_type_line = $source_line;
 
-$basetypes = <BASE>;
+my $basetypes = <BASE>;
 $basesource_line += ($basetypes =~ tr/\n//);;
 
-$types = <>;
+my $types = <>;
 $source_line += ($types =~ tr/\n//);;
 
-print "#line $base_source_line \"runtimebase.in\"\n";
+print "#line $base_source_line \"$srcTemplateDir/runtimebase.in\"\n";
 $baseheader = <BASE>;
 print $baseheader;
 $basesource_line += ($baseheader =~ tr/\n//);;
@@ -187,13 +207,14 @@ read_types($basetypes, "runtimebase.in", $basesource_type_line);
 read_types($types, "$prefix.in", $source_type_line);
 
 ### Begining of `$prefix.h'
+my @header;
 push @header, $autogenerated;
 # TODO: Capitalize prefix
-push @header, "#ifndef " . $prefix . "_H\n";
-push @header, "#define " . $prefix . "_H\n";
+push @header, "#pragma once\n";
 push @header, "namespace run {\n";
 
 my $count = 0;
+my @builtin;
 while (<>) {
   my ($comments,$type,$name,$cname,$params,$code) =
     m|^((?:\s*//[^\n]*\n)*) # comment lines
@@ -225,7 +246,7 @@ while (<>) {
       assoc_error("$prefix.in", $source_line, $type);
   }
   my @asy_params = asy_params($params, "$prefix.in", $source_line);
-  push @builtin, "#line $source_line \"$prefix.in\"\n"
+  push @builtin, "#line $source_line \"$srcTemplateDir/$prefix.in\"\n"
       . "  addFunc(ve, run::" . $cname
       . ", " . $type_map{$type}
       . ", " . symbolize($name)
@@ -237,22 +258,22 @@ while (<>) {
   # Build REGISTER_BLTIN command for builtin functions which are not added to
   # the environment.
   if (not $name and $cname) {
-    push @builtin, "#line $source_line \"$prefix.in\"\n"
+    push @builtin, "#line $source_line \"$srcTemplateDir/$prefix.in\"\n"
       . "  REGISTER_BLTIN(run::" . $cname
       . ',"' . $cname . '"' . ");\n";
   }
 
   # Handle marshalling of values to/from stack
-  $qualifier = ($type eq "item" ? "" : "<$type>");
+  my $qualifier = ($type eq "item" ? "" : "<$type>");
   $code =~ s/\breturn ([^;]*);/{$stack->push$qualifier($1); return;}/g;
-  $args = join("",c_params(@params));
+  my $args = join("",c_params(@params));
 
   print $comments;
-  $ncomments = ($comments =~ tr/\n//);
+  my $ncomments = ($comments =~ tr/\n//);
   $source_line += $ncomments;
-  print "#line $source_line \"$prefix.in\"\n";
+  print "#line $source_line \"$srcTemplateDir/$prefix.in\"\n";
   my $prototype=$type . " " . $name . "(" . $params . ");";
-  $nprototype = ($prototype =~ tr/\n//)+1;
+  my $nprototype = ($prototype =~ tr/\n//)+1;
   $source_line += $nprototype;
   if($name) {
     clean_params($prototype);
@@ -261,7 +282,7 @@ while (<>) {
   print "void $cname(stack *";
   if($type ne "void" or $params ne "") {print $stack;}
   print ")\n{\n$args";
-  print "#line $source_line \"$prefix.in\"";
+  print "#line $source_line \"$srcTemplateDir/$prefix.in\"";
   print "$code}\n\n";
 
   $source_line -= $ncomments+$nprototype;
@@ -279,19 +300,21 @@ print "} // namespace trans\n";
 
 ### End of `header.h'
 push @header, "}\n\n";
-push @header, "#endif // ". $prefix . "_H\n";
 
 undef $/;
-open HEADER, "<", "$prefix.h";
-$orig_header = <HEADER>;
-$new_header = join "", @header;
+open my $HEADER, "<", $outHeaderFile;
+my $orig_header = <$HEADER>;
+close $HEADER;
+
+my $new_header = join "", @header;
 if ($new_header ne $orig_header) {
-	open HEADER, ">", "$prefix.h";
-	print HEADER $new_header;
+	open $HEADER, ">", $outHeaderFile;
+	print $HEADER $new_header;
+    close $HEADER;
 }
 
 if ($errors) {
-  unlink("$prefix.h");
-  unlink("$prefix.cc");
+  unlink($outHeaderFile);
+  unlink($outSrcFile);
 }
 exit($errors);
