@@ -39,10 +39,6 @@ void endwait(pthread_cond_t& signal, pthread_mutex_t& lock);
 }
 
 namespace settings {
-extern const char PROGRAM[];
-extern const char VERSION[];
-extern const char BUGREPORT[];
-
 extern char *argv0;
 
 void Warn(const string& s);
@@ -94,7 +90,9 @@ char *getArg(int n);
 
 Int getScroll();
 
+#if !defined(_MSC_VER)
 extern mode_t mask;
+#endif
 
 bool xe(const string& texengine);
 bool lua(const string& texengine);
